searchNodes=[{"type":"module","title":"Jackalope","doc":"Jackalope is an opinionated MQTT library that simplifies the use of Tortoise311 MQTT with cloud IoT services. Jackalope aims to make an interface that: Ensure that important messages are delivered to the broker, by having a local &quot;post office&quot; and tracking the in flight messages, and implementing a concept of ttl (time to live) on the messages placed in the mailbox; ensuring the &quot;request to unlock the door&quot; won't happen two hours later when the MQTT connection finally reconnects. This allows Jackalope to accept publish requests while the connection is down. Besides this Jackalope aims to provide helpers for local testing, allowing you to test your application without having a connection to AWS; Jackalope should take care of that. Usage The Jackalope module implements a start_link/1 function; use this to start Jackalope as part of your application supervision tree. If properly supervised it will allow you to start and stop Jackalope with the part the application that needs MQTT connectivity. Jackalope is configured using a keyword list, consult the Jackalope.start_link/1 documentation for information on the available option values. Once Jackalope is running it is possible to publish messages to the broker; in addition to this there are some connection specific functionality is exposed, allowing us to ask for the connection status, and request a connection reconnect. Subscriptions are static only and are set as part of the connection options provided to Jackalope. Jackalope.publish(topic, payload) will publish a message to the MQTT broker; Jackalope.reconnect() will disconnect from the broker and reconnect; this is useful if the device changes network connection. Please see the documentation for each of the functions for more information on usage; publish functions accept options such as setting quality of service and time to live.","ref":"Jackalope.html"},{"type":"function","title":"Jackalope.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Jackalope.html#child_spec/1"},{"type":"function","title":"Jackalope.connection_status/0","doc":"Get the current MQTT connection status","ref":"Jackalope.html#connection_status/0"},{"type":"function","title":"Jackalope.publish/3","doc":"Publish a message to the MQTT broker The payload will get published on topic . Jackalope will keep the message in a queue until we got a connection, at which point it will dispatch the publish. This of course present us with a problem: what if we place a publish request to &quot;unlock the front door&quot; while the client is offline? We don't want to receive a message that the front door has been unlocked two hours later when the MQTT client reconnect; To solve that problem we have a ttl option we can specify on the publish. Jackalope . publish ( &quot;doors/front_door&quot; , %{ action : &quot;unlock&quot; } , qos : 1 , ttl : 5_000 ) The available publish options are: qos (default 1 ) sets the quality of service of the message delivery; Notice that only quality of service 0 an 1 are supported by AWS IoT. retain (default false ) sets whether the broker should retain the message. Note that AWS IoT does not support this feature. ttl (default 3_600_000 ) sets how long (in msecs) publishing the message will be retried until it has expired.","ref":"Jackalope.html#publish/3"},{"type":"function","title":"Jackalope.reconnect/0","doc":"Request the MQTT client to reconnect to the broker This can be useful on devices that has multiple network interfaces.","ref":"Jackalope.html#reconnect/0"},{"type":"function","title":"Jackalope.start_link/1","doc":"Start a Jackalope session This will start a supervised group of processes; part of the group will keep track of the topic filter subscription state, and hold a list of yet to be published messages, as well as the requested subscription changes; the other part of the process tree will keep the MQTT connection specific parts, making sure we got a connection. See the main documentation on the Jackalope module for more information on the process architecture. Jackalope.start_link/1 takes a keyword list containing option values, that configure the instance, as an argument. The available options and their defaults are: client_id (default: &quot;jackalope&quot;), string that will be used as the client_id of the MQTT connection; see t Tortoise311.client_id for more information on valid client ids. Notice that the client_id needs to be unique on the server, so two clients may not have the same client_id. user_name (optional) specifies the MQTT Username. password (optional) specifies the MQTT Password. Google cloud IOT requires a JWT for this field. initial_topics (optional) specifies a list of topic_filters Jackalope should connect to when a connection has been established. Notice that this list is also used should a reconnect happen later in the life-cycle. Note that Jackalope does not support dynamic subscriptions or unsubscribing. This is the only mechanism for subscribing. clean_session (default: true), a flag bit that controls the life cycle of the session state. The MQTT session lasts only as long as the network connection between client and server. When the session ends, all session state in the server is deleted. handler (default: Jackalope.Handler.Logger ) specifies the module implementing the callbacks (implementing Jackalope.Handler behaviour) to use. This module reacts to the events Jackalope communicates about the connection life-cycle, including receiving a message on a subscribed topic filter. Read the documentation for Jackalope.Handler for more information on the events and callbacks. server (default: {Tortoise311.Transport.Tcp, [host: &quot;localhost&quot;, port: 1883]}) specifies the connection type, and its options, to use when connecting to the MQTT server. The default specification will attempt to connect to a broker running on localhost:1883, on an insecure connection. This value should only be used for testing and development. Server options for use with AWS IoT: [ verify: :verify_peer, host: mqtt_host(), # must return the full name, without wild cards , for e.g. &quot;abcdefghijklmo-ats.iot.us-east-1.amazonaws.com&quot; port: mqtt_port(), # must return the correct port, e.g. 443 alpn_advertised_protocols: [&quot;x-amzn-mqtt-ca&quot;], server_name_indication: to_charlist(mqtt_host()), cert: cert, # the device's X509 certificate in DER format key: key, # the device's private key in DER format cacerts: [signer_cert] ++ aws_ca_certs(), # the device's signer cert, plus AWS IoT CA certs in DER format to be returned by aws_ca_certs() versions: [:&quot;tlsv1.2&quot;], customize_hostname_check: [match_fun: :public_key.pkix_verify_hostname_match_fun(:https)] ] work_list_mod names the module implementing the Jackalope WorkList protocol that will be used to manage the publish commands sent to Tortoise by the Jackalope Session. The module must also implement the function @spec new(function(), function(), non_neg_integer(), Keyword.t()) :: any() . See Jackalope.TransientWorkList (the default) for examples. max_work_list_size (default: 100) specifies the maximum number of unexpired work orders Jackalope will retain in its work list (the commands yet to be sent to the MQTT server). When the maximum is reached, the oldest work order is dropped before adding a new work order. last_will (default: nil) specifies the last will message that should get published on the MQTT broker if the connection is closed or dropped unexpectedly. If we want to specify a last will topic we should define a keyword list containing the following: topic (Required) the topic to post the last will message to; this should be specified as a string and it should be a valid MQTT topic; consult t Tortoise311.topic for more info on valid MQTT topics. payload (default: nil) the payload of the last will message; qos (default: 0) either 0 or 1, denoting the quality of service the last will message should get published with; note that QoS=2 is not supported by AWS IoT. backoff (default: [min_interval: 1_000, max_interval: 30_000]) gives the bounds of an exponential backoff algorithm used when retrying from failed connections.","ref":"Jackalope.html#start_link/1"},{"type":"behaviour","title":"Jackalope.Handler","doc":"Behaviour defining callbacks triggered during the MQTT life-cycle The jackalope handler is stateless, so if state is needed one could route the messages to stateful processes, and inform the system about connection and subscription state. Most of the callbacks are optional.","ref":"Jackalope.Handler.html"},{"type":"callback","title":"Jackalope.Handler.connection/1","doc":"Called when the MQTT connection changes status This can be used to inform other parts of the system about the state of the connection; possible values are :up and :down , where up means that the MQTT client has a connection to the broker; down means that the connection has been dropped.","ref":"Jackalope.Handler.html#c:connection/1"},{"type":"callback","title":"Jackalope.Handler.handle_error/1","doc":"Handle errors produced by Jackalope that should be reacted to During the connection life-cycle various errors can occur, and while Jackrabbit and Tortoise311 will try to correct the situation, some errors require user intervention. The optional handle_error/1 callback can help inform the surrounding system of errors. @impl Jackalope.Handler def handle_error ( { :publish_error , work_order , :ttl_expired } ) do Logger . error ( &quot;Work order expired: \#{ inspect ( work_order ) } &quot; ) end def handle_error ( _otherwise ) do _ignore = nil end If this callback is implemented one should make sure to make a catch-all to prevent unhandled errors from crashing the handler.","ref":"Jackalope.Handler.html#c:handle_error/1"},{"type":"callback","title":"Jackalope.Handler.handle_message/2","doc":"Called when receiving a message matching one of the subscriptions The callback will receive two arguments; the MQTT topic in list form, where each of the topic levels are an item. This allows us to pattern match on topic filters with wildcards.","ref":"Jackalope.Handler.html#c:handle_message/2"},{"type":"callback","title":"Jackalope.Handler.last_will/0","doc":"Produces the last will message for the current connection, or nil if the last will in the connection options is to be used Example: [topic: hub_serial_number/message&quot;, payload: %{code: &quot;going_down&quot;, msg: &quot;Last will message&quot;}, qos: 1]","ref":"Jackalope.Handler.html#c:last_will/0"},{"type":"callback","title":"Jackalope.Handler.subscription/2","doc":"Called when a topic filter subscription state changes This can be used to inform other parts of the system that we should (or shouldn't) expect messages received on the given topic_filter . The status values are :up and :down , where up means that the broker has accepted a subscription request to the specific topic_filter , and down means that the broker has accepted an unsubscribe request.","ref":"Jackalope.Handler.html#c:subscription/2"},{"type":"type","title":"Jackalope.Handler.last_will/0","doc":"","ref":"Jackalope.Handler.html#t:last_will/0"},{"type":"type","title":"Jackalope.Handler.payload/0","doc":"","ref":"Jackalope.Handler.html#t:payload/0"},{"type":"type","title":"Jackalope.Handler.topic/0","doc":"","ref":"Jackalope.Handler.html#t:topic/0"},{"type":"type","title":"Jackalope.Handler.topic_filter/0","doc":"","ref":"Jackalope.Handler.html#t:topic_filter/0"},{"type":"type","title":"Jackalope.Handler.topic_levels/0","doc":"","ref":"Jackalope.Handler.html#t:topic_levels/0"},{"type":"module","title":"Jackalope.Handler.Logger","doc":"A Jackalope.Handler that logs everything it sees This handler will be used by Jackalope if no handler has been specified in the option list passed to Jackalope.start_link/1 .","ref":"Jackalope.Handler.Logger.html"},{"type":"module","title":"Jackalope.TransientWorkList","doc":"A simple work list that does not survive across reboots.","ref":"Jackalope.TransientWorkList.html"},{"type":"function","title":"Jackalope.TransientWorkList.new/4","doc":"Create a new work list","ref":"Jackalope.TransientWorkList.html#new/4"},{"type":"type","title":"Jackalope.TransientWorkList.t/0","doc":"","ref":"Jackalope.TransientWorkList.html#t:t/0"},{"type":"protocol","title":"Jackalope.WorkList","doc":"A work list manages work items that have yet to be sent out for processing, or have been sent out and are waiting for confirmation of completion.","ref":"Jackalope.WorkList.html"},{"type":"function","title":"Jackalope.WorkList.count/1","doc":"Count the number of work items","ref":"Jackalope.WorkList.html#count/1"},{"type":"function","title":"Jackalope.WorkList.count_pending/1","doc":"Count the number of pending work items","ref":"Jackalope.WorkList.html#count_pending/1"},{"type":"function","title":"Jackalope.WorkList.done/2","doc":"Remove an item from the pending set","ref":"Jackalope.WorkList.html#done/2"},{"type":"function","title":"Jackalope.WorkList.empty?/1","doc":"Are there work items","ref":"Jackalope.WorkList.html#empty?/1"},{"type":"function","title":"Jackalope.WorkList.peek/1","doc":"Get the current item on the work list, if any. Leave it on the work list.","ref":"Jackalope.WorkList.html#peek/1"},{"type":"function","title":"Jackalope.WorkList.pending/2","doc":"Move a work item to the pending set as it is being worked on, awaiting confirmation of completion.","ref":"Jackalope.WorkList.html#pending/2"},{"type":"function","title":"Jackalope.WorkList.pop/1","doc":"Remove the current item from the work list","ref":"Jackalope.WorkList.html#pop/1"},{"type":"function","title":"Jackalope.WorkList.push/2","doc":"Push an item on the work list","ref":"Jackalope.WorkList.html#push/2"},{"type":"function","title":"Jackalope.WorkList.remove_all/1","doc":"Remove all work items","ref":"Jackalope.WorkList.html#remove_all/1"},{"type":"function","title":"Jackalope.WorkList.reset_pending/1","doc":"Empty out the set of pending items","ref":"Jackalope.WorkList.html#reset_pending/1"},{"type":"type","title":"Jackalope.WorkList.item/0","doc":"","ref":"Jackalope.WorkList.html#t:item/0"},{"type":"type","title":"Jackalope.WorkList.t/0","doc":"All the types that implement this protocol.","ref":"Jackalope.WorkList.html#t:t/0"},{"type":"type","title":"Jackalope.WorkList.work_list/0","doc":"","ref":"Jackalope.WorkList.html#t:work_list/0"},{"type":"module","title":"Jackalope.WorkList.Expiration","doc":"Common functions for work lists.","ref":"Jackalope.WorkList.Expiration.html"},{"type":"function","title":"Jackalope.WorkList.Expiration.after?/2","doc":"Does the first expiration come after the second?","ref":"Jackalope.WorkList.Expiration.html#after?/2"},{"type":"function","title":"Jackalope.WorkList.Expiration.expiration/1","doc":"","ref":"Jackalope.WorkList.Expiration.html#expiration/1"},{"type":"function","title":"Jackalope.WorkList.Expiration.now/0","doc":"","ref":"Jackalope.WorkList.Expiration.html#now/0"},{"type":"function","title":"Jackalope.WorkList.Expiration.rebase_expiration/3","doc":"Recalculate an old expiration given the latest time before stop and the earliest time after restart. Assumes restart happened soon after stopping.","ref":"Jackalope.WorkList.Expiration.html#rebase_expiration/3"},{"type":"function","title":"Jackalope.WorkList.Expiration.unexpired?/2","doc":"","ref":"Jackalope.WorkList.Expiration.html#unexpired?/2"},{"type":"type","title":"Jackalope.WorkList.Expiration.expiration/0","doc":"","ref":"Jackalope.WorkList.Expiration.html#t:expiration/0"},{"type":"extras","title":"Changelog","doc":"","ref":"changelog.html"},{"type":"extras","title":"Changelog - v0.7.2","doc":"Updates Remove Logger warnings for Elixir 1.15 Fixes Add :clean_session option support. This fixes issues with QoS 1 sessions","ref":"changelog.html#v0-7-2"},{"type":"extras","title":"Changelog - v0.7.2","doc":"Updates Support querying the session's connection status","ref":"changelog.html#v0-7-2"},{"type":"extras","title":"Changelog - v0.7.1","doc":"Fixes Added dialyzer flags :missing_return, :extra_return Minor fixes to remove new dialyzer warnings Updated CI to OTP 25","ref":"changelog.html#v0-7-1"},{"type":"extras","title":"Changelog - v0.7.0","doc":"Updated BREAKING CHANGE: Jackalope no longer encodes or decodes MQTT message payloads.","ref":"changelog.html#v0-7-0"},{"type":"extras","title":"Changelog - v0.6.0","doc":"Updates BREAKING CHANGE: Dynamic subscriptions are no longer supported :infinity is no longer the default for publish work item TTL or work list bound BREAKING CHANGE: The API for publishing is simplified; there's now only one publish call Fixes Dynamic subscriptions were broken in failure and retry scenarios, removing support for them fixes that","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"Changelog - v0.5.1","doc":"Updates Uses backoff options","ref":"changelog.html#v0-5-1"},{"type":"extras","title":"Changelog - v0.5.0","doc":"Updates Bumped tortoise311 to 0.11","ref":"changelog.html#v0-5-0"},{"type":"extras","title":"Changelog - v0.4.2 - DEPRECATED","doc":"Updates All references to Tortoise are replaced by Tortoise311","ref":"changelog.html#v0-4-2-deprecated"},{"type":"extras","title":"Changelog - v0.4.1","doc":"Updates Now using tortoise311 forked from tortoise","ref":"changelog.html#v0-4-1"},{"type":"extras","title":"Changelog - v0.4.0","doc":"Updates Session work list can be bounded to a set size","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"Changelog - v0.3.0","doc":"Updates Moved to Tortoise 0.10.0 Last-will message callbacks","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"Changelog - v0.2.0","doc":"Bug fixes Fix topic return type for :payload_decode_error - This is a breaking change if you're using it. Reduce logging when everything is working fine","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"Changelog - v0.1.0","doc":"Initial release to hex.","ref":"changelog.html#v0-1-0"}]